import {Component, Inject, PLATFORM_ID} from '@angular/core'
import {isPlatformBrowser} from '@angular/common'
import {AppState} from './modules/shared/app-state'
import {SubscribingComponent} from './modules/shared/components/subscribing.component'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from './modules/shared/service/error.service'
import {ConfigService} from './modules/shared/service/config.service'
import {Config} from './modules/shared/model/config.model'
import {environment} from '../environments/environment'

@Component({
  selector: 'cre-root',
  templateUrl: './app.component.html',
  styleUrls: ['./app.component.sass']
})
export class AppComponent extends SubscribingComponent {
  isOnline: boolean
  isServerOnline = true
  favIcon: HTMLLinkElement = document.querySelector('#favicon')

  constructor(
    @Inject(PLATFORM_ID) private platformId: object,
    private configService: ConfigService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    this.isOnline = isPlatformBrowser(this.platformId)
    super.ngOnInit()

    this.subscribe(
      this.appState.serverOnline$,
      online => this.isServerOnline = online
    )

    this.favIcon.href = environment.apiUrl + "/file?path=images%2Ficon"
  }

  reload() {
    window.location.reload()
  }
}
