import {Component} from '@angular/core'
import {Material} from '../../../../shared/model/material.model'
import {MaterialService} from '../../../../material/service/material.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../../shared/components/subscribing.component'
import {MixService} from '../../../services/mix.service'
import {ErrorService} from '../../../../shared/service/error.service'

@Component({
  selector: 'cre-mix-add',
  templateUrl: './mix-add.component.html',
  styleUrls: ['./mix-add.component.sass']
})
export class MixAddComponent extends ErrorHandlingComponent {
  recipeId: number | null
  materials: Material[] | null

  constructor(
    private materialService: MaterialService,
    private mixService: MixService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    super.ngOnInit()

    this.recipeId = this.urlUtils.parseIntUrlParam('recipeId')

    this.subscribe(
      this.materialService.getAllForMixCreation(this.recipeId),
      m => this.materials = m
    )
  }

  submit(values) {
    this.subscribeAndNavigate(
      this.mixService.saveWithUnits(values.name, values.recipeId, values.materialTypeId, values.mixMaterials, values.units),
      `/color/edit/${this.recipeId}`
    )
  }
}
