import {Component} from '@angular/core'
import {GroupService} from '../../services/group.service'
import {Group, Permission} from '../../../shared/model/user'
import {AccountService} from '../../../accounts/services/account.service'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-groups',
  templateUrl: './list.component.html',
  styleUrls: ['./list.component.sass']
})
export class ListComponent extends ErrorHandlingComponent {
  groups$ = this.groupService.all
  defaultGroup: Group = null
  columns = [
    {def: 'name', title: 'Nom', valueFn: g => g.name},
    {def: 'permissionCount', title: 'Nombre de permissions', valueFn: g => g.permissions.length}
  ]
  buttons = [{
    text: 'Définir par défaut',
    clickFn: group => this.setDefaultGroup(group),
    disabledFn: group => this.isDefaultGroup(group)
  }, {
    text: 'Modifier',
    linkFn: group => `/admin/group/edit/${group.id}`,
    permission: Permission.EDIT_USERS
  }]

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'nodefaultgroup',
    consumer: () => this.alertService.pushWarning('Aucun groupe par défaut n\'a été défini sur cet ordinateur')
  }]

  constructor(
    private groupService: GroupService,
    private accountService: AccountService,
    private alertService: AlertService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Groupes'
  }

  ngOnInit(): void {
    this.subscribe(
      this.groupService.defaultGroup,
      group => this.defaultGroup = group,
      true
    )
  }

  setDefaultGroup(group: Group) {
    this.subscribe(
      this.groupService.setDefaultGroup(group),
      () => this.defaultGroup = group,
      true
    )
  }

  isDefaultGroup(group: Group): boolean {
    return this.defaultGroup && this.defaultGroup.id == group.id
  }
}
