import {Injectable} from '@angular/core'
import {ApiService} from '../../shared/service/api.service'
import {Observable} from 'rxjs'
import {User, Group, Permission} from '../../shared/model/user'
import {tap} from 'rxjs/operators'

@Injectable({
  providedIn: 'root'
})
export class GroupService {
  constructor(
    private api: ApiService
  ) {
  }

  get all(): Observable<Group[]> {
    return this.api.get<Group[]>('/user/group')
  }

  get allWithDefault(): Observable<Group[]> {
    return this.all
      .pipe(tap(groups => groups.unshift({
        id: -1,
        name: 'Aucun',
        permissions: []
      })))
  }

  getById(id: number): Observable<Group> {
    return this.api.get<Group>(`/user/group/${id}`)
  }

  get defaultGroup(): Observable<Group> {
    return this.api.get<Group>('/user/group/default')
  }

  setDefaultGroup(value: Group): Observable<void> {
    return this.api.post<void>(`/user/group/default/${value.id}`, {})
  }

  getUsersForGroup(id: number): Observable<User[]> {
    return this.api.get<User[]>(`/user/group/${id}/users`)
  }

  addUserToGroup(id: number, user: User): Observable<void> {
    return this.api.put<void>(`/user/group/${id}/${user.id}`)
  }

  removeUserFromGroup(user: User): Observable<void> {
    return this.api.delete<void>(`/user/group/${user.group.id}/${user.id}`)
  }

  save(name: string, permissions: Permission[]): Observable<Group> {
    const group = {name, permissions}
    return this.api.post<Group>('/user/group', group)
  }

  update(id: number, name: string, permissions: Permission[]): Observable<Group> {
    const group = {id, name, permissions}
    return this.api.put<Group>('/user/group', group)
  }

  delete(id: number): Observable<Group> {
    return this.api.delete<Group>(`/user/group/${id}`)
  }
}
