import {Injectable} from '@angular/core'
import {User} from './model/user'
import {Subject} from 'rxjs'
import {Title} from '@angular/platform-browser'

@Injectable({
  providedIn: 'root'
})
export class AppState {
  private readonly KEY_AUTHENTICATED = 'authenticated'
  private readonly KEY_AUTHENTICATION_EXPIRATION = 'authentication-expiration'
  private readonly KEY_LOGGED_IN_USER = 'logged-in-user'

  authenticatedUser$ = new Subject<{ authenticated: boolean, authenticatedUser: User }>()
  serverOnline$ = new Subject<boolean>()

  constructor(
    private titleService: Title
  ) {
  }

  resetAuthenticatedUser() {
    this.isAuthenticated = false
    this.authenticationExpiration = -1
    this.authenticatedUser = null
  }

  set isServerOnline(isOnline: boolean) {
    if (!isOnline) {
      this.authenticatedUser = null
    }
    this.serverOnline$.next(isOnline)
  }

  get isAuthenticated(): boolean {
    return sessionStorage.getItem(this.KEY_AUTHENTICATED) === 'true'
  }

  set isAuthenticated(value: boolean) {
    sessionStorage.setItem(this.KEY_AUTHENTICATED, value.toString())
    this.authenticatedUser$.next({
      authenticated: value,
      authenticatedUser: this.authenticatedUser
    })
  }

  get authenticationExpiration(): number {
    return parseInt(sessionStorage.getItem(this.KEY_AUTHENTICATION_EXPIRATION))
  }

  set authenticationExpiration(value: number) {
    sessionStorage.setItem(this.KEY_AUTHENTICATION_EXPIRATION, value.toString())
  }

  get authenticatedUser(): User {
    const userString = sessionStorage.getItem(this.KEY_LOGGED_IN_USER)
    return userString ? JSON.parse(userString) : null
  }

  set authenticatedUser(value: User) {
    if (value === null) {
      sessionStorage.removeItem(this.KEY_LOGGED_IN_USER)
    } else {
      sessionStorage.setItem(this.KEY_LOGGED_IN_USER, JSON.stringify(value))
    }
    this.authenticatedUser$.next({
      authenticated: this.isAuthenticated,
      authenticatedUser: value
    })
  }

  set title(value: string) {
    this.titleService.setTitle(`CRE: ${value}`)
  }
}
