import {Form, FormControl} from '@angular/forms'
import {filterMap} from '../utils/map.utils'

export class Config {
  static readonly INSTANCE_NAME = 'instance.name'
  static readonly INSTANCE_LOGO_PATH = 'instance.logo.path'
  static readonly INSTANCE_ICON_PATH = 'instance.icon.path'
  static readonly INSTANCE_URL = 'instance.url'
  static readonly DATABASE_URL = 'database.url'
  static readonly DATABASE_USER = 'database.user'
  static readonly DATABASE_PASSWORD = 'database.password'
  static readonly DATABASE_VERSION = 'database.version.supported'
  static readonly RECIPE_APPROBATION_EXPIRATION = 'recipe.approbation.expiration'
  static readonly TOUCH_UP_KIT_CACHE_PDF = 'touchupkit.pdf.cache'
  static readonly TOUCH_UP_KIT_EXPIRATION = 'touchupkit.expiration'
  static readonly EMERGENCY_MODE = 'env.emergency'
  static readonly BACKEND_BUILD_VERSION = 'env.build.version'
  static readonly BACKEND_BUILD_TIME = 'env.build.time'
  static readonly JAVA_VERSION = 'env.java.version'
  static readonly OPERATING_SYSTEM = 'env.os'

  static readonly IMAGE_CONFIG_KEYS = [
    Config.INSTANCE_LOGO_PATH,
    Config.INSTANCE_ICON_PATH
  ]

  public key: string
  public requireRestart: boolean
  public editable: boolean
  public content?: string
  public lastUpdated?: string
}

export class ConfigKeyContent {
  public key: string
  public content: string
}

export function filterConfigKeyControlMap(map: Map<string, FormControl>): Map<string, FormControl> {
  return filterMap(map, (key, control) => {
    return control.dirty &&
      Config.IMAGE_CONFIG_KEYS.indexOf(key) < 0 &&  // Filter image configs because they are sent to a different endpoint
      control.value !== undefined &&
      control.value !== null
  })
}

export function filterImageConfigKeyControlMap(map: Map<string, FormControl>): Map<string, FormControl> {
  return filterMap(map, (key, control) => {
    return Config.IMAGE_CONFIG_KEYS.indexOf(key) >= 0 && control.dirty
  })
}

export function mapToConfigKeyContent(key: string, control: FormControl): ConfigKeyContent {
  return {key, content: control.value}
}

export function mapToConfigKeyContentArray(map: Map<string, FormControl>): ConfigKeyContent[] {
  const array: ConfigKeyContent[] = []
  map.forEach((control, key) => {
    array.push(mapToConfigKeyContent(key, control))
  })
  return array
}
