export class User {
  constructor(
    public id: number,
    public firstName: string,
    public lastName: string,
    public explicitPermissions: Permission[],
    public permissions: Permission[],
    public group?: Group,
    public lastLoginTime?: Date
  ) {
  }
}

export class Group {
  constructor(
    public id: number,
    public name: string,
    public permissions: Permission[]
  ) {
  }
}

export enum Permission {
  VIEW_RECIPES = 'VIEW_RECIPES',
  VIEW_USERS = 'VIEW_USERS',
  VIEW_CATALOG = 'VIEW_CATALOG',

  EDIT_RECIPES_PUBLIC_DATA = 'EDIT_RECIPES_PUBLIC_DATA',
  EDIT_RECIPES = 'EDIT_RECIPES',
  EDIT_MATERIALS = 'EDIT_MATERIALS',
  EDIT_MATERIAL_TYPES = 'EDIT_MATERIAL_TYPES',
  EDIT_COMPANIES = 'EDIT_COMPANIES',
  EDIT_USERS = 'EDIT_USERS',
  EDIT_CATALOG = 'EDIT_CATALOG',

  VIEW_TOUCH_UP_KITS = 'VIEW_TOUCH_UP_KITS',
  EDIT_TOUCH_UP_KITS = 'EDIT_TOUCH_UP_KITS',

  PRINT_MIXES = 'PRINT_MIXES',
  ADD_TO_INVENTORY = 'ADD_TO_INVENTORY',
  DEDUCT_FROM_INVENTORY = 'DEDUCT_FROM_INVENTORY',

  ADMIN = 'ADMIN'
}

export const mapped_permissions = {
  view: [
    {permission: Permission.VIEW_RECIPES, description: 'Voir les recettes', impliedPermissions: []},
    {permission: Permission.VIEW_CATALOG, description: 'Voir le catalogue', impliedPermissions: []},
    {permission: Permission.VIEW_USERS, description: 'Voir les utilisateurs', impliedPermissions: []},
    {permission: Permission.VIEW_TOUCH_UP_KITS, description: 'Voir les kits de retouche', impliedPermissions: []}
  ],
  edit: [{
    permission: Permission.EDIT_RECIPES_PUBLIC_DATA,
    description: 'Modifier les données publiques des recettes',
    impliedPermissions: [Permission.VIEW_RECIPES]
  }, {
    permission: Permission.EDIT_RECIPES,
    description: 'Modifier les recettes',
    impliedPermissions: [Permission.EDIT_RECIPES_PUBLIC_DATA]
  }, {
    permission: Permission.EDIT_MATERIALS,
    description: 'Modifier les produits',
    impliedPermissions: [Permission.VIEW_CATALOG]
  }, {
    permission: Permission.EDIT_MATERIAL_TYPES,
    description: 'Modifier les types de produit',
    impliedPermissions: [Permission.VIEW_CATALOG]
  }, {
    permission: Permission.EDIT_COMPANIES,
    description: 'Modifier les bannières',
    impliedPermissions: [Permission.VIEW_CATALOG]
  }, {
    permission: Permission.EDIT_CATALOG,
    description: 'Modifier le catalogue',
    impliedPermissions: [Permission.EDIT_MATERIALS, Permission.EDIT_MATERIAL_TYPES, Permission.EDIT_COMPANIES]
  }, {
    permission: Permission.EDIT_USERS,
    description: 'Modifier les utilisateurs',
    impliedPermissions: [Permission.VIEW_USERS]
  }, {
    permission: Permission.EDIT_TOUCH_UP_KITS,
    description: 'Modifier les kits de retouche',
    impliedPermissions: [Permission.VIEW_TOUCH_UP_KITS]
  }],
  other: [{
    permission: Permission.PRINT_MIXES,
    description: 'Imprimer les mélanges (bPac)',
    impliedPermissions: [Permission.VIEW_RECIPES]
  }, {
    permission: Permission.ADD_TO_INVENTORY,
    description: 'Ajouter dans l\'inventaire',
    impliedPermissions: [Permission.VIEW_CATALOG]
  }, {
    permission: Permission.DEDUCT_FROM_INVENTORY,
    description: 'Déduire dans l\'inventaire',
    impliedPermissions: [Permission.VIEW_RECIPES]
  }, {
    permission: Permission.ADMIN,
    description: 'Administrateur',
    impliedPermissions: [Permission.EDIT_RECIPES, Permission.EDIT_CATALOG, Permission.EDIT_USERS, Permission.EDIT_TOUCH_UP_KITS, Permission.PRINT_MIXES, Permission.ADD_TO_INVENTORY, Permission.DEDUCT_FROM_INVENTORY]
  }]
}
