import {Injectable} from '@angular/core'
import {Config, filterConfigKeyControlMap, filterImageConfigKeyControlMap, mapToConfigKeyContentArray} from '../model/config.model'
import {Observable} from 'rxjs'
import {ApiService} from './api.service'
import {FormControl} from '@angular/forms'
import {transformMap} from '../utils/map.utils'

@Injectable({
  providedIn: 'root'
})
export class ConfigService {
  constructor(
    private api: ApiService
  ) {
  }

  get(key: string): Observable<Config> {
    return this.api.get<Config>(`/config/${key}`)
  }

  set(configs: Map<string, FormControl>): Observable<void> {
    const body = mapToConfigKeyContentArray(filterConfigKeyControlMap(configs))
    const imageConfigs = filterImageConfigKeyControlMap(configs)

    this.setImages(imageConfigs)
    return this.api.put<void>('/config', body)
  }

  setImage(key: string, image: File): Observable<void> {
    const body = new FormData()
    body.append('key', key)
    body.append('image', image)

    return this.api.put<void>('/config/image', body)
  }

  restart(): Observable<void> {
    return this.api.post<void>('/config/restart')
  }

  private setImages(configs: Map<string, FormControl>) {
    const subscriptions = this.getImageConfigsSubscriptions(configs)
    while (subscriptions.length > 0) {
      const subscription = subscriptions.pop().subscribe({
        next: () => subscription.unsubscribe()
      })
    }
  }

  private getImageConfigsSubscriptions(configs: Map<string, FormControl>): Observable<void>[] {
    return transformMap(configs, (key, control) => {
      return this.setImage(key, control.value)
    })
  }
}
