import {Component, Input, OnInit, ViewChild, ViewEncapsulation} from '@angular/core'
import {TouchUpKitProduct} from '../../shared/model/touch-up-kit.model'
import {MatTable} from '@angular/material/table'

@Component({
  selector: 'touchupkit-product-editor',
  templateUrl: 'product-editor.html',
  styleUrls: ['product-editor.sass'],
  encapsulation: ViewEncapsulation.None
})
export class TouchUpKitProductEditor implements OnInit {
  productCols = ['name', 'description', 'quantity', 'removeButton']

  @Input() products: TouchUpKitProduct[]

  @ViewChild(MatTable) table: MatTable<TouchUpKitProduct>
  hoveredProduct: TouchUpKitProduct | null
  selectedProduct: TouchUpKitProduct | null

  ngOnInit(): void {
    if (!this.products) {
      this.products = [this.emptyProduct]
    }
  }

  addRow() {
    const newProduct = this.emptyProduct

    this.products.push(newProduct)
    this.table.renderRows()
    this.selectedProduct = newProduct
  }

  removeRow(product: TouchUpKitProduct) {
    this.products = this.products.filter(p => p !== product)
    this.table.renderRows()
  }

  isFocused(index: number, product: TouchUpKitProduct): boolean {
    return (!this.hoveredProduct && index === 0) ||
      this.hoveredProduct === product ||
      this.selectedProduct === product
  }

  get empty(): boolean {
    return this.products.length <= 0
  }

  get emptyProduct(): TouchUpKitProduct {
    return {
      id: null,
      name: '',
      description: '',
      quantity: 1,
      ready: false
    }
  }
}
